# base for attentionpoint views

from analyses.analysisbase import forms as app_forms
from analyses.analysisbase import tables as app_tables
from analyses.analysisbase.views.scenario import localnavigationlist
from analyses.analysisbase.views import helpers
from analyses.main import analyses_navigationlist

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from installations.views.main import installations_navigationlist

from utils import crud
from utils import forms as utils_forms
from utils.versioning import versionedcrud
from utils.autocomplete import autocomplete
from utils.versioning.session import get_current_version

class MixIn(object):
    "Common items for object views"
##    appconfig = app_models.AppConfig
##    model = appconfig.get_model('attentionpoint')
##    parent_class = appconfig.get_model('measure')
    parent_field = 'measure'
    parent_tab = 'attentionpoints'

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['currentanalysistype'] = self.appconfig.analysistype_short

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.AttentionPointSheet, model=self.model)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self, extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)
        if object.suggestionlist:
            extra_context['breadcrumbs'] = [
                {'url': object.suggestionlist.get_absolute_url(),
                'label': u'%s' % object.suggestionlist}]
        elif object.measure:
            extra_context['breadcrumbs'] = [{'url': object.measure.get_absolute_url('attentionpoints'),
                'label': u'%s' % object.measure}]

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = None
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.AttentionPointForm, model=self.model)

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = None
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.AttentionPointSheet, model=self.model)

class report(MixIn, versionedcrud.ReportObjectView):
    pass

class add_to_measure(MixIn, versionedcrud.AddObjectView):
    form_class = None
    parent_tab = 'attentionpoints'
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.AttentionPointForm, model=self.model)

class add_to_apsuggestionlist(add_to_measure):
    def __new__(cls, *args, **kwargs):
        cls.parent_class = cls.appconfig.get_model('apsuggestionlist')
        cls.parent_field = 'suggestionlist'
        return super(add_to_apsuggestionlist, cls).__new__(cls, *args, **kwargs)

class compare(MixIn, versionedcrud.CompareObjectView):
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.AttentionPointSheet, model=self.model)
        super(compare, self).setup_from_args(*args, **kwargs)

def type_lookup(request, appconfig):
    model = appconfig.get_model('attentionpoint')
    queryset = model.objects.values('type').order_by('type').distinct()
    return autocomplete(request, queryset, 'type', 'type')

def name_lookup(request, appconfig):
    model = appconfig.get_model('attentionpoint')
    (version, special) = get_current_version(request)
    queryset = model.objects.get_versionedobjects(version=version).values('name').order_by('name')
    return autocomplete(request, queryset, 'name', 'name')

class AttentionPointFilter(crud.Filter):
    form_class= None
##    model = app_models.DegradationProtectionLayer

    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
                {'filteritem': 'type', 'parameter': 'type', 'type': 'normal'},
            ]
    def get_form(self):
        if not self.form_class:
            return utils_forms.create_class_with_meta(app_forms.AttentionPointFilterForm, model=self.model)
        return self.form_class

class AttentionPointListObject(helpers.TableListObject):
    defaultsort = 'name'
    default_table_class = app_tables.AllAttentionPointsTable
    table_classes = ['attentionpoints']

    def get_title(self):
        return u'%s %s' % (_('Attention points'), self.appconfig.analysistype)

    def get_query_set(self):
        return self.model.objects.filter(suggestionlist__isnull=True).exclude(SL=True).get_versionedobjects(self.version)

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('attentionpoints'):
            q = queryset.filter(description__isnull=False)
            if q:
                html_strings.append(u'<h2>%s</h2>\n' % _('Attention point details'))
                for item in q:
                    html_strings.extend(item.get_report_body(user, language, choices, subreport=True))

    def add_css(self, css_files, queryset, user, language, choices):
        css_files.extend(self.model.get_listreport_css(language, choices))
        if choices.get('attentionpoints'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class attentionpoints(helpers.TableListView):
    "List of attention points"

    defaultsort = 'name'
    paginate_by = 100
    redirect_on_unique = True
    default_csv_table_class = app_tables.AllAttentionPointsCSVTable
    allow_pdf = True
    allow_csv = True
    pdfsendmail = True
    pdfselectform_class = utils_forms.ListReportForm

    default_table_class = app_tables.AllAttentionPointsTable
##    model = app_models.DegradationAttentionPoint
##    filter = AttentionPointFilter()

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=self.model)

    def get_title(self):
        return u'%s %s' % (_('Attention points'), self.appconfig.analysistype)

    def get_query_set(self, request):
        return self.model.objects.filter(suggestionlist__isnull=True).exclude(SL=True).get_versionedobjects(self.version)

    def add_to_extra_context(self, extra_context):
        super(attentionpoints, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  analyses_navigationlist(self.appconfig.namespace)
        extra_context['tablist'] =  self.get_navigationlist(self.appconfig.namespace)
        extra_context['currenttab'] = 'attentionpoints'
        extra_context['breadcrumbs'] = [{'label': _('Lists'), 'url': u''}]

    def get_navigationlist(self, namespace):
        return localnavigationlist(namespace)

