# Deviation scenario views

from analyses.deviations import models as app_models
from analyses.deviations import forms as app_forms
from analyses.deviations import tables as app_tables
from analyses.deviations.views.scenario import deviations_localnavigationlist
from analyses.deviations import evaluation
from analyses.main import analyses_navigationlist

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from utils import crud
from utils import forms as utils_forms
from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion

class display(crud.DisplayObjectView):
    model = app_models.DeviationLopaPath
    allow_change = False
    allow_delete = False
    form_class = None
    template = "deviations/scenario_path.html"
    tabs = []

    def add_to_extra_context(self, extra_context, object):
        crud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        breadcrumbs = [
            {'url': object.scenario.get_absolute_url('lopa'),
                'label': u'%s' % object.scenario}]
        extra_context['breadcrumbs'] = breadcrumbs

class report(crud.ReportObjectView):
    model = app_models.DeviationLopaPath

class LopaPathFilter(crud.Filter):
    form = app_forms.LopaPathFilterForm
    filters = [
                {'filteritem': 'high', 'parameter': 'high', 'type': 'boolean'},
            ]

class LopaPathListObject(versionedcrud.TableListObject):
    defaultsort = 'name'
    table_class = app_tables.DeviationLopaPathTable
    queryset = app_models.DeviationLopaPath.objects.order_by('scenario', 'number')
    title = _('LOPA scenarios Control of deviation')
    table_classes = ['lopapaths']

    def get_query_set(self):
        if self.version==WorkVersion:
            return self.queryset.filter(scenario__version__isnull=True)
        elif self.version:
            return self.queryset.filter(scenario__version=self.version)
        else:
            return self.queryset.none()

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('lopapathdetails'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        css_files.extend(app_models.DeviationLopaPath.get_listreport_css(language, choices))
        if choices.get('lopapathdetails'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class lopapaths(versionedcrud.TableListView):
    "List of lopa paths"

    table_class = app_tables.DeviationLopaPathTable
    defaultsort = 'number'
    queryset = app_models.DeviationLopaPath.objects.order_by('scenario', 'number')
    paginate_by = 100
    title = _('LOPA scenarios Control of deviation')
    filter = LopaPathFilter()
    allow_pdf = True
    listobject = LopaPathListObject()
    pdfselectform_class = utils_forms.BareListReportForm
    pdfsendmail = True

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.DeviationLopaPath)

    def get_query_set(self, request):
        if self.version==WorkVersion:
            return self.queryset.filter(scenario__version__isnull=True).select_related()
        elif self.version:
            return self.queryset.filter(scenario__version=self.version).select_related()
        else:
            return self.queryset.none()

    def setup_from_args(self, *args, **kwargs):
        super(lopapaths, self).setup_from_args(*args, **kwargs)
        scenarios = app_models.DeviationScenario.objects.filter(use_LOPA=True).get_versionedobjects(self.version)
        for scenario in scenarios:
            errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)

    def add_to_extra_context(self, extra_context):
        super(lopapaths, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  analyses_navigationlist('deviations')
        extra_context['tablist'] =  self.get_navigationlist('deviations')
        extra_context['currenttab'] = 'lopapaths'
        extra_context['breadcrumbs'] = [{'label': _('Lists'), 'url': u''}]

    def get_navigationlist(self, namespace):
        return deviations_localnavigationlist(namespace)

views = {
    'display': display(),
    'report': report(),
    'lopapaths': lopapaths(),
}
